@@ -29,6 +29,8 @@ from output_templates import output_templates
 from metadata import metaqueries
 from plothandler import Plot
 from sqlpython import Parser
+import warnings
+warnings.filterwarnings('ignore', 'BaseException.message', DeprecationWarning)
 try:
     import pylab
 except (RuntimeError, ImportError):
@@ -259,6 +261,27 @@ def centeredSlice(lst, center=0, width=1):
     else:
         return lst[max(center-width,0):center+width+1] 
     
+
+def offset_to_line(instring, offset):
+    r"""
+    >>> offset_to_line('abcdefghijkl', 5)
+    (0, 'abcdefghijkl', 5)
+    >>> offset_to_line('ab\ncd\nefg', 6)
+    (2, 'efg', 0)
+    >>> offset_to_line('ab\ncd\nefg', 5)
+    (1, 'cd\n', 2)
+    >>> offset_to_line('abcdefghi\njkl', 5)
+    (0, 'abcdefghi\n', 5)
+    >>> offset_to_line('abcdefghi\njkl', 700)
+    
+    """
+    lineNum = 0
+    for line in instring.splitlines(True):
+        if offset < len(line):
+            return lineNum, line, offset
+        lineNum += 1
+        offset -= len(line)
+        
 class sqlpyPlus(sqlpython.sqlpython):
     defaultExtension = 'sql'
     abbrev = True    
@@ -313,6 +336,15 @@ class sqlpyPlus(sqlpython.sqlpython):
             return '', '', ''
         return cmd, arg, line
 
+    def perror(self, err, statement=None):
+        try:
+            linenum, line, offset = offset_to_line(statement.parsed.raw, err.message.offset)
+            print line.strip()
+            print '%s*' % (' ' * offset)
+            print 'ERROR at line %d:' % (linenum + 1)
+        except AttributeError:
+            pass
+        print str(err)
     def dbms_output(self):
         "Dumps contents of Oracle's DBMS_OUTPUT buffer (where PUT_LINE goes)"
         try:
@@ -691,10 +723,12 @@ class sqlpyPlus(sqlpython.sqlpython):
             self.pfeedback('\nSelected Max Num rows (%d)' % self.rc)
         
     def do_cat(self, arg):
-        '''Shortcut for SELECT * FROM'''
+        '''Shortcut for SELECT * FROM
         return self.do_select(self.parsed('SELECT * FROM %s;' % arg, 
                                           terminator = arg.parsed.terminator or ';', 
-                                          suffix = arg.parsed.suffix))
+                                          suffix = arg.parsed.suffix))'''
+        return self.onecmd('SELECT * FROM %s%s%s' % (arg, arg.parsed.terminator or ';',
+                                                     arg.parsed.suffix or ''))
 
     def _pull(self, arg, opts, vc=None):
         """Displays source code."""
@@ -1351,8 +1385,11 @@ class sqlpyPlus(sqlpython.sqlpython):
         sortdirection = (hasattr(opts, 'reverse') and opts.reverse and 'DESC') or 'ASC'
         orderby = 'object_type %s, object_name %s' % (sortdirection, sortdirection)
         if hasattr(opts, 'timesort') and opts.timesort:
-            orderby = 'last_ddl_time %s, %s' % (
-                ('ASC' if hasattr(opts, 'reverse') and opts.reverse else 'DESC'), orderby)
+            if hasattr(opts, 'reverse') and opts.reverse:
+                direction = 'DESC'
+            else:
+                direction = 'ASC'
+            orderby = 'last_ddl_time %s, %s' % (direction, orderby)
         clauses['orderby'] = orderby    
         statement = '''
             SELECT object_type || '/' || %(owner)s object_name AS name %(moreColumns)s 
